import pgindex
from django.core.management.base import BaseCommand, CommandError
from django.utils.translation import ugettext as _
from optparse import make_option
from pgindex.models import Index


class Command(BaseCommand):
    help = _('Reindex for django-pgindex')

    option_list = BaseCommand.option_list + (
        make_option('--apps',
            action='store',
            dest='apps',
            default='',
            help=_('specify apps to reindex for.'),
            ),
        make_option('--all',
            action='store_true',
            dest='all',
            default=False,
            help=_('reindex all apps.'),
            )
        )
    def handle(self, *args, **options):
        registry = pgindex.helpers._registry
        if options['all']:
            Index._default_manager.all().delete()
        elif options['apps']:
            apps = [ app.strip() for app in options['apps'].split(',') ]
            Index._default_manager.filter(obj_app_label__in=apps).delete()
        else:
            raise CommandError(_('No apps to reindex.'))
        for model, idx_cls in registry.iteritems():
            app_label = model._meta.app_label
            if options['all'] or app_label in apps:
                print _('Reindexing %s') % app_label,
                for obj in model._default_manager.all():
                    idx = idx_cls(obj)
                    idx.update()
                print '\tOK'




from django.db import models


class TSVectorField(models.Field):
    def __init__(self, *args, **kwargs):
        kwargs.update({
            'null': True,
            'editable': False,
            'serialize': False,
        })
        super(TSVectorField, self).__init__(*args, **kwargs)

    def get_prep_value(self, value):
        """
        It is not possible to set the vector value this way, you need to
        execute SQL using UPDATE.
        """
        return None

    def db_type(self, connection):
        return 'tsvector'

    def south_field_triple(self):
        from south.modelsinspector import introspector
        cls = self.__class__
        dot_name = '%s.%s' % (cls.__module__ , cls.__name__)
        args, kwargs = introspector(self)
        return (dot_name, args, kwargs)




#coding=utf-8
import datetime
import re
from django.db.models import signals
from django.utils.encoding import force_unicode
from pgindex.models import Index


_registry = {}
non_alpha_pat = re.compile(r'[\.\^\$\*\+\?\{\[\]\}\\\\|\(\)!"#%&/=\',;:]')


def register(model, search_cls):
    _registry[model] = search_cls
    signals.post_save.connect(update_index, sender=model)
    signals.post_delete.connect(delete_index, sender=model)


def delete_index(sender, instance, **kwargs):
    idx = Index.objects.get_for_object(instance)
    if idx:
        idx.delete()


def update_index(sender, instance, **kwargs):
    index_cls = _registry[sender]
    idx = index_cls(instance)
    idx.update()


def search(q, weight=None, dictionary='simple'):
    """
    Helper function to search the index
    """
    if weight is None:
        weight = weight or '{0.1, 0.2, 0.4, 1.0}' # default weight in postgres
    extra = {
        'select': {
            'rank': ("ts_rank_cd('%s', ts, plainto_tsquery(%%s), 0)" % weight)
        },
        'select_params': (q,),
        'where': ("ts @@ plainto_tsquery('%s', %%s)" % dictionary,),
        'params': (q,),
        'order_by': ('-rank',)
    }
    return Index.publ.extra(**extra)


class Vector(object):
    weight = 'B'
    dictionary = 'simple'
    clean = True

    def __init__(self, value, weight=None, dictionary=None, clean=None):
        if weight is not None:
            self.weight = weight
        if dictionary is not None:
            self.dictionary = dictionary
        if clean is not None:
            self.clean = clean
        if self.clean is True:
            # clean out html, django template vars, django comments
            for pat in [r'<[^>]*?>', r'{{[^}]*?}}', r'{#[^#}]*?#}']:
                value = re.sub(pat, '', value)
            # clean out not characters, ' and % can break stuff
            value = non_alpha_pat.sub('', value)
        self.value = value

    @property
    def tsvector(self):
        return u"setweight(to_tsvector('%s', E'%s'), '%s')" % (
            self.dictionary, self.value, self.weight
            )




import datetime
import re
from django.utils.encoding import force_unicode
from abc import ABCMeta, abstractmethod
from django.db import connection, transaction
from django.utils.encoding import force_unicode
from django.utils.html import strip_tags
from pgindex.models import Index


class IndexBase(object):
    __meta__ = ABCMeta

    def __init__(self, obj):
        self.obj = obj

    def get_title(self):
        """
        Get the object index title.
        """
        return self.obj.title

    def get_description(self):
        """
        Get the object index description.
        """
        return ''

    def get_url(self):
        """
        Get the object url.
        """
        return self.obj.get_absolute_url()

    def get_image(self):
        """
        Get an image for the object.
        """
        return ''

    def get_data(self):
        """
        This additional data will be pickled and stored in the index model. It
        could be anything that you want to access from the index, for example
        the model instance it self.
        """
        return None

    def get_publish(self):
        """
        ``False`` means not to index this object at all
        """
        return True

    def get_start_publish(self):
        """
        Controls published stop datetime for the index.
        ``None`` means the beginning of time.
        """
        return None

    def get_stop_publish(self):
        """
        Controls published stop datetime for the index.
        ``None`` means the end of time.
        """
        return None

    @abstractmethod
    def get_vectors(self):
        """
        This is weher you return your search vectors. It should be an iterable
        with instances of ``pgindex.helpers.Vector``.
        """
        raise NotImplemented

    def get_tsvector(self):
        tsvectors = [ v.tsvector for v in self.get_vectors() ]
        return u' || '.join(tsvectors)

    def update(self):
        stop_publish = self.get_stop_publish()
        now = datetime.datetime.now()
        if (not self.get_publish() or stop_publish and stop_publish < now):
            # no point in indexing this object
            idx = Index.objects.get_for_object(self.obj)
            if idx:
                idx.delete()
            return
        idx = Index.objects.get_for_object(self.obj, create=True)
        idx.title = self.get_title()
        idx.description = self.get_description()
        idx.image = self.get_image()
        idx.url = self.get_url()
        idx.data = self.get_data()
        idx.start_publish = self.get_start_publish()
        idx.stop_publish = stop_publish
        idx.save()
        idx.set_ts(self.get_tsvector())




from setuptools import setup, find_packages
from setuptools.command.test import test


class TestCommand(test):
    def run(self):
        from tests.runtests import runtests
        runtests()


setup(
    name='django-pgindex',
    version='0.7.0',
    description='Search for Django and PostgreSQL',
    long_description=open('README.rst').read(),
    author='Mikko Hellsing',
    author_email='mikko@aino.se',
    license='BSD',
    url='https://github.com/aino/django-pgindex',
    platforms='any',
    packages=find_packages(exclude=['tests', 'tests.*']),
    zip_safe=False,
    cmdclass={"test": TestCommand},
    install_requires=[
        'django-stringfield>=0.1.5',
        'django-cerial>=0.0.2',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Framework :: Django',
    ],
)




#coding=utf-8
import datetime
from cerial import PickleField
from django.db import models, connection, transaction
from django.db.models import Q, signals
from django.utils.encoding import force_unicode
from django.utils.encoding import smart_str
from pgindex.fields import TSVectorField
from stringfield import StringField


class IndexManager(models.Manager):
    def get_for_object(self, obj, create=False):
        opts = obj._meta
        params = {
            'obj_app_label': force_unicode(opts.app_label),
            'obj_model_name': force_unicode(opts.object_name),
            'obj_pk': force_unicode(obj.pk),
            }
        try:
            return self.get_query_set().get(**params)
        except self.model.DoesNotExist:
            if create:
                params.update({
                    'obj_model_verbose_name': force_unicode(opts.verbose_name),
                    'obj_model_verbose_name_plural': force_unicode(opts.verbose_name_plural),
                })
                return self.create(**params)

class IndexPublManager(IndexManager):
    def get_query_set(self):
        qs = super(IndexPublManager, self).get_query_set()
        now = datetime.datetime.now()
        return qs.filter(
            (Q(start_publish__isnull=True) | Q(start_publish__lte=now)) &
            (Q(stop_publish__isnull=True) | Q(stop_publish__gte=now))
            )

class Index(models.Model):
    title = StringField()
    description = models.TextField(blank=True)
    url = StringField()
    image = models.ImageField(upload_to='index', blank=True)
    data = PickleField(null=True)
    ts = TSVectorField()
    start_publish = models.DateTimeField(db_index=True, null=True)
    stop_publish = models.DateTimeField(db_index=True, null=True)
    # generic foreign key kind of.
    obj_app_label = StringField()
    obj_model_name = StringField()
    obj_pk = StringField()

    obj_model_verbose_name = StringField()
    obj_model_verbose_name_plural = StringField()

    objects = IndexManager()
    publ = IndexPublManager()

    def get_absolute_url(self):
        return self.url

    def set_ts(self, ts):
        """
        Performs *raw* update on ts
        """
        ts = smart_str(ts)
        sql = "UPDATE %s SET ts = %s WHERE id = %s;" % (
            self._meta.db_table, ts, self.pk
            )
        cursor = connection.cursor()
        cursor.execute(sql)
        transaction.commit_unless_managed()

    class Meta:
        unique_together = (('obj_app_label', 'obj_model_name', 'obj_pk'),)


def create_index(app, created_models, verbosity, **kwargs):
    if Index in created_models:
        opts = Index._meta
        sql = "CREATE INDEX pgindex_ts_idx ON %s USING gin(ts)" % opts.db_table
        cursor.execute(sql)
        transaction.commit_unless_managed()

signals.post_syncdb.connect(
    create_index,
    sender=models,
    dispatch_uid="pgindex.models.create_index"
    )

